@@ -1,12 +1,16 @@
 import logging
 import requests
-from datetime import datetime, time
+from dateutil import parser
+from datetime import datetime
 from dataclasses import dataclass
 from requests.auth import HTTPBasicAuth
-from typing import Optional, List, Dict, Set, Any
+from typing import Optional, List, Dict, Any
 from openhab_pythonrule_engine.cache import Cache
 
 
+logging = logging.getLogger(__name__)
+
+
 @dataclass
 class Item:
     item_name: str
@@ -31,20 +35,7 @@ class Item:
 
     def get_state_as_datetime(self) -> datetime:
         text = self.get_state_as_text()
-        try:
-            # time_string may be in format 21:30
-            return datetime.strptime(text, '%H:%M')
-        except:
-            try:
-                # time_string may be in format "2017-07-14T21:30:00.000+0200"
-                dt, timezone = text.split(".")
-                return datetime.strptime(dt, '%Y-%m-%dT%H:%M:%S')
-            except:
-                try:
-                    # time_string seems to be in format "2017-07-14T21:30:00"
-                    return datetime.strptime(text, '%Y-%m-%dT%H:%M:%S')
-                except:
-                    return None
+        return datetime.fromtimestamp(parser.parse(text).timestamp())
 
 
 @dataclass
@@ -69,15 +60,17 @@ class TextItem(Item):
         else:
             return float(self.value)
 
-    def serialize(self, value) -> Optional[str]:
-        if self.value is None:
+    def serialize(self, value_to_serialize) -> Optional[str]:
+        if value_to_serialize is None:
             return None
-        elif type(value) == bool:
-            return "ON" if value else "OFF"
-        elif type(value) == datetime:
-            return value.strftime('%Y-%m-%dT%H:%M:%S')
+        elif type(value_to_serialize) == bool:
+            return "ON" if value_to_serialize else "OFF"
+        elif type(value_to_serialize) == datetime:
+            return value_to_serialize.strftime('%Y-%m-%dT%H:%M:%S')
+        elif type(value_to_serialize) == int:
+            return str(float(value_to_serialize))
         else:
-            return str(value)
+            return str(value_to_serialize)
 
 
 @dataclass
@@ -103,14 +96,16 @@ class NumericItem(Item):
             return ""
         return str(self.value)
 
-    def serialize(self, value) -> Optional[str]:
-        if self.value is None:
+    def serialize(self, value_to_serialize) -> Optional[str]:
+        if value_to_serialize is None:
             return None
         else:
-            if type(value) == bool:
-                return "1" if value else "0"
+            if type(value_to_serialize) == bool:
+                return "1" if value_to_serialize else "0"
+            elif type(value_to_serialize) == int:
+                return str(float(value_to_serialize))
             else:
-                return str(value)
+                return str(value_to_serialize)
 
 
 @dataclass
@@ -140,15 +135,15 @@ class BooleanItem(Item):
         else:
             return str(self.value)
 
-    def serialize(self, value) -> Optional[str]:
-        if type(value) == bool:
-            return "ON" if value else "OFF"
-        elif type(value) == float:
-            return "ON" if value == 1.0 else "OFF"
-        elif type(value) == int:
-            return "ON" if value == 1 else "OFF"
+    def serialize(self, value_to_serialize) -> Optional[str]:
+        if type(value_to_serialize) == bool:
+            return "ON" if value_to_serialize else "OFF"
+        elif type(value_to_serialize) == float:
+            return "ON" if value_to_serialize == 1.0 else "OFF"
+        elif type(value_to_serialize) == int:
+            return "ON" if value_to_serialize == 1 else "OFF"
         else:
-            return "ON" if (str(value).lower() in ["true", "on"]) else "OFF"
+            return "ON" if (str(value_to_serialize).lower() in ["true", "on"]) else "OFF"
 
 
 def to_item(data) -> Optional[Item]:
@@ -240,7 +235,10 @@ class ItemRegistry:
             logging.warning("error occurred by calling " + uri, e)
 
     def has_item(self, item_name: str) -> bool:
-        return self.get_item(item_name) != None
+        if item_name is None:
+            return False
+        else:
+            return self.get_item(item_name) != None
 
     def get_group_membernames(self, group_name) -> List[str]:
         return [item.item_name for item in self.get_items().values() if group_name in item.group_names]
@@ -296,34 +294,27 @@ class ItemRegistry:
     def get_state_as_datetime(self, item_name: str, datetime_string: str="1970-01-01") -> datetime:
         state = self.get_item(item_name)
         if state is None or state.value is None:
-            return self.__to_datetime_object(datetime_string)
+            return datetime.fromtimestamp(parser.parse(datetime_string).timestamp())
         else:
             return state.get_state_as_datetime()
 
-    def __to_datetime_object(self, datetime_string: str):
-        try:
-            # time_string may be in format 21:30
-            return datetime.strptime(datetime_string, '%H:%M')
-        except:
-            try:
-                # time_string may be in format "2017-07-14T21:30:00.000+0200"
-                dt, timezone = datetime_string.split(".")
-                return datetime.strptime(dt, '%Y-%m-%dT%H:%M:%S')
-            except:
-                # time_string seems to be in format "2017-07-14T21:30:00"
-                return datetime.strptime(datetime_string, '%Y-%m-%dT%H:%M:%S')
 
-    def set_state(self, item_name: str, new_state):
+    def set_state(self, item_name: str, new_state) -> bool:
         if new_state is None:
             logging.warning("try to set " + item_name + " = None. ignoring it")
-            return
         else:
             item_metadata = self.get_item_metadata(item_name)
             if item_metadata is None:
                 raise Exception("item " + item_name + " not exists")
             else:
                 old_state = self.get_state(item_name, None)
-                if old_state  != new_state:
-                    self.set_item_state(item_name, item_metadata.serialize(new_state))
-                    logging.debug("set " + item_name + " = " + str(new_state))
-
+                serialized_old_sate = item_metadata.serialize(old_state)
+                serialized_new_state = item_metadata.serialize(new_state)
+                if serialized_old_sate != serialized_new_state:
+                    try:
+                        self.set_item_state(item_name, serialized_new_state)
+                        logging.debug("set " + item_name + " = " + serialized_new_state)
+                        return True
+                    except Exception as e:
+                        logging.warning("could not set " + item_name + " = " + serialized_new_state, e)
+        return False
